@@ -12,7 +12,7 @@ use feature ();
 package Zydeco;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.519';
+our $VERSION   = '0.520';
 
 use Keyword::Simple ();
 use PPR;
@@ -36,6 +36,25 @@ BEGIN {
 		my ($me, $caller) = @_;
 		!!$gather{$me}{$caller};
 	}
+	sub _predeclare {
+		my ($me, $opts, $kind, $pkg, $pkgopts) = @_;
+		# Figure out type name
+		return if $kind =~ /role_generator/;
+		my %opts = (%$opts, %$pkgopts);
+		my $qname = 'MooX::Press'->qualify_name($pkg, $opts{'prefix'}, $opts{'extends'}//$opts{'_stack'}[-1]);
+		
+		if ($kind =~ /class_generator/) {
+			my $typename1 = $opts{'class_type_name'}
+				|| sprintf('%sClass', 'MooX::Press'->type_name($qname, $opts{'prefix'}));
+			my $typename2 = $opts{'instance_type_name'}
+				|| sprintf('%sInstance', 'MooX::Press'->type_name($qname, $opts{'prefix'}));
+			'Zydeco'->_predeclare($opts{'caller'}, $opts{'type_library'}, $typename1, $typename2);
+		}
+		else {
+			my $typename = $opts{'type_name'} || 'MooX::Press'->type_name($qname, $opts{'prefix'});
+			'Zydeco'->_predeclare($opts{'caller'}, $opts{'type_library'}, $typename);
+		}
+	}
 	sub import {
 		my ($me, $action, $caller) = (shift, shift, scalar caller);
 		if ($action eq -gather) {
@@ -50,6 +69,7 @@ BEGIN {
 						}
 					}
 					push @{ $gather{$me}{$caller}{$kind}||=[] }, $pkg, $v;
+					$me->_predeclare( $gather{$me}{$caller}, $kind, $pkg, $v );
 				}
 				else {
 					$gather{$me}{$caller}{$k} = $v;
@@ -1469,6 +1489,33 @@ sub unimport {
 	goto \&Exporter::Tiny::unimport;
 }
 
+sub _predeclare {
+	my ($me, $caller, $types, @names) = @_;
+	
+	for my $name (@names) {
+		my $cached;
+		my $T = sub () {
+			if ( !$cached or $cached->isa('Type::Tiny::_DeclaredType') ) {
+				my $got = $types->can('get_type') && $types->get_type($name);
+				$cached = $got if $got;
+				$cached ||= 'Type::Tiny::_DeclaredType'->new(
+					name    => $name,
+					library => $types,
+				);
+			}
+			$cached;
+		};
+		eval qq{
+			package Zydeco; # allow namespace::autoclean to clean them
+			no warnings 'redefine';
+			*$caller\::$name        = \$T;
+			*$caller\::is_$name     = sub (\$) { \$T->()->check(\@_) };
+			*$caller\::assert_$name = sub (\$) { \$T->()->assert_return(\@_) };
+			1;
+		} or die($@);
+	}
+}
+
 sub import {
 	no warnings 'closure';
 	my ($me, %opts) = (shift, @_);
@@ -1488,14 +1535,7 @@ sub import {
 	#
 	if ($opts{declare}) {
 		my $types = $opts{type_library};
-		for my $name (@{ $opts{declare} }) {
-			eval qq{
-				sub $caller\::$name         ()   { goto \\&$types\::$name }
-				sub $caller\::is_$name      (\$) { goto \\&$types\::is_$name }
-				sub $caller\::assert_$name  (\$) { goto \\&$types\::assert_$name }
-				1;
-			} or die($@);
-		}
+		$me->_predeclare($caller, $types, @{ $opts{declare} });
 	}
 	
 	# Export utility stuff
@@ -1517,7 +1557,7 @@ sub import {
 	my @libs = qw/ Types::Standard Types::Common::Numeric Types::Common::String /;
 	push @libs, $opts{type_library} if $opts{type_library}->isa('Type::Library');
 	for my $library (@libs) {
-		$library->import::into($caller, $_)
+		$library->import::into($caller, { replace => 1 }, $_)
 			for grep $want{$_}, qw( -types -is -assert );
 	}
 	
@@ -2282,7 +2322,7 @@ sub _include {
 #{
 #	package Zydeco::Anonymous::Package;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.519';
+#	our $VERSION   = '0.520';
 #	use overload q[""] => sub { ${$_[0]} }, fallback => 1;
 #	sub DESTROY {}
 #	sub AUTOLOAD {
@@ -2293,7 +2333,7 @@ sub _include {
 #	
 #	package Zydeco::Anonymous::Class;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.519';
+#	our $VERSION   = '0.520';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	sub new {
 #		my $me = shift;
@@ -2306,12 +2346,12 @@ sub _include {
 #	
 #	package Zydeco::Anonymous::Role;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.519';
+#	our $VERSION   = '0.520';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	
 #	package Zydeco::Anonymous::ParameterizableClass;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.519';
+#	our $VERSION   = '0.520';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	sub generate_package {
 #		my $me  = shift;
@@ -2325,7 +2365,7 @@ sub _include {
 #
 #	package Zydeco::Anonymous::ParameterizableRole;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.519';
+#	our $VERSION   = '0.520';
 #	our @ISA       = qw(Zydeco::Anonymous::Package);
 #	sub generate_package {
 #		my $me  = shift;
